class RightAscension
{
   var hrs : int;
   var min : int;
   var sec : int;
   
   function IsValidHMS(h: int, m: int, s: int) : bool 
   {
        h >= 0 && h < 24 &&
        m >= 0 && m < 60 &&
        s >= 0 && s < 60                    
   }   
      
   function IsValid() : bool 
   reads this;
   {
       IsValidHMS(hrs, min, sec)                
   }
   
   function method Get_Hrs() : int
   reads this;
   requires IsValid();
   ensures Get_Hrs() == hrs; 
   { hrs }  
   
   method Set_Hrs(h: int)
   modifies this;
   requires IsValid() && h >= 0 && h < 24;
   ensures IsValid() && hrs == h && old(min) == min && old(sec) == sec;
   {
       hrs := h;
   }
         
   function method Get_Min() : int
   reads this;
   requires IsValid();
   ensures Get_Min() == min; 
   { min }  
   
   method Set_Min(m: int)
   modifies this;
   requires IsValid() && m >= 0 && m < 24;
   ensures IsValid() && min == m && old(hrs) == hrs && old(sec) == sec;
   {
       min := m;
   }
   
   function method Get_Sec() : int
   reads this;
   requires IsValid();
   ensures Get_Sec() == sec; 
   { sec }  
   
   method Set_Sec(s: int)
   modifies this;
   requires IsValid() && s >= 0 && s < 24;
   ensures IsValid() && sec == s && old(min) == min && old(hrs) == hrs;
   {
       sec := s;
   }

   function method ToTenthDeg() : int
   reads this;
   requires IsValid();
   {    
		(150*hrs) + (5*min/2)
   }
   
   method Init(h: int, m: int, s:int)
   modifies this;
   requires IsValidHMS(h, m, s);
   ensures IsValid();
   {
       hrs := h;
       min := m;
       sec := s;   
   }           
}

class Declination
{
   var wdg : int;
   var min : int;
   var sec : int;
     
   function IsValidDMS(d: int, m: int, s: int) : bool 
   {
        d >= -90 && d <= 90 &&
        m >= 0 && m < 60 &&
        s >= 0 && s < 60 &&
       ((d == 90 || d == -90) ==> (m == 0 && s == 0))                           
   }
   
   function IsValid() : bool 
   reads this;
   {
       IsValidDMS(wdg, min, sec)                
   }
   
   function method Get_WDeg() : int
   reads this;
   requires IsValid();
   ensures Get_WDeg() == wdg; 
   { wdg }  
   
   method Set_WDeg(w: int)
   modifies this;
   requires IsValid() && IsValidDMS(w, min, sec);
   ensures IsValid() && wdg == w && old(min) == min && old(sec) == sec;
   {
       wdg := w;
   }
         
   function method Get_Min() : int
   reads this;
   requires IsValid();
   ensures Get_Min() == min; 
   { min }  
   
   method Set_Min(m: int)
   modifies this;
   requires IsValid() && IsValidDMS(wdg, m, sec);
   ensures IsValid() && min == m && old(wdg) == wdg && old(sec) == sec;
   {
       min := m;
   }
   
   function method Get_Sec() : int
   reads this;
   requires IsValid();
   ensures Get_Sec() == sec; 
   { sec }  
   
   method Set_Sec(s: int)
   modifies this;
   requires IsValid() && IsValidDMS(wdg, min, s);
   ensures IsValid() && sec == s && old(min) == min && old(wdg) == wdg;
   {
       sec := s;
   }

   function method ToTenthDeg() : int
   reads this;
   requires IsValid();
   { 
       if (wdg < 0) then
		(10*wdg) - (min/6)
	   else
		(10*wdg) + (min/6)      
   }
   
   method Init(w: int, m: int, s:int)
   modifies this;
   requires IsValidDMS(w, m, s);
   ensures IsValid();
   {
       wdg := w;
       min := m;
       sec := s;   
   }              
}


class CelestialPoint
{
	ghost var Repr: set<object>;
	var alpha : RightAscension;
	var delta : Declination;

	function IsValid() : bool
	reads this, alpha, delta;
	{
	     this in Repr &&
		 alpha in Repr &&
		 delta in Repr &&
	     alpha != null && delta != null && alpha.IsValid() && delta.IsValid()
	}

   function method Get_Alpha() : RightAscension
   reads this;
   requires IsValid();
   ensures Get_Alpha() == alpha; 
   { alpha }  
   
   method Set_Alpha(a: RightAscension)
   modifies this;
   requires a != null && a.IsValid() && IsValid();
   ensures IsValid() && old(delta) == delta;
   {
       Repr := (Repr - {alpha}) + {a};
       alpha := a;
   }

   function method Get_Delta() : Declination
   reads this;
   requires IsValid();
   ensures Get_Delta() == delta; 
   { delta }  
   
   method Set_Delta(d: Declination)
   modifies this;
   requires d != null && d.IsValid() && IsValid();
   ensures IsValid() && old(alpha) == alpha;
   {
       Repr := (Repr - {delta}) + {d};
       delta := d;
   }		

   method Init(a: RightAscension, d : Declination)
   modifies this;
   requires a != null && d != null && a.IsValid() && d.IsValid();
   ensures IsValid();
   {
        alpha := a;
		delta := d;
		Repr  := {};
		Repr  := Repr + {this} + {a} + {d};
   }
}